@@ -8,8 +8,12 @@ use std::{
 
 use anyhow::{Context, Result};
 use clap::{ArgEnum, Args};
+use rayon::prelude::*;
 use sha1::{Digest, Sha1};
 use swc_common::SourceMap;
+use tempdir::TempDir;
+
+use crate::{util::all_js_files, CREDUCE_INPUT_ENV_VAR, CREDUCE_MODE_ENV_VAR};
 
 #[derive(Debug, Args)]
 pub struct ReduceCommand {
@@ -17,6 +21,10 @@ pub struct ReduceCommand {
 
     #[clap(long, arg_enum)]
     pub mode: ReduceMode,
+
+    /// If true, the input file will be removed after the reduction.
+    #[clap(long)]
+    pub remove: bool,
 }
 
 #[derive(Debug, Clone, Copy, ArgEnum)]
@@ -27,24 +35,45 @@ pub enum ReduceMode {
 
 impl ReduceCommand {
     pub fn run(self, _cm: Arc<SourceMap>) -> Result<()> {
-        fs::copy(&self.path, "input.js").context("failed to copy")?;
+        let js_files = all_js_files(&self.path)?;
+
+        js_files
+            .into_par_iter()
+            .map(|path| self.reduce_file(&path))
+            .collect::<Result<Vec<_>>>()?;
+
+        Ok(())
+    }
+
+    fn reduce_file(&self, src_path: &Path) -> Result<()> {
+        let dir = TempDir::new("dbg-swc").context("failed to create a temp directory")?;
+
+        let input = dir.path().join("input.js");
+
+        fs::copy(&src_path, &input).context("failed to copy")?;
 
         let mut c = Command::new("creduce");
 
         c.env(
-            "CREDUCE_COMPARE",
+            CREDUCE_MODE_ENV_VAR,
             match self.mode {
                 ReduceMode::Size => "SIZE",
                 ReduceMode::Semantics => "SEMANTICS",
             },
         );
+        c.env(CREDUCE_INPUT_ENV_VAR, &input);
+
         let exe = current_exe()?;
         c.arg(&exe);
-        c.arg("input.js");
+        c.arg(&input);
         let status = c.status().context("failed to run creduce")?;
 
         if status.success() {
-            move_to_data_dir("input.js".as_ref())?;
+            move_to_data_dir(&input)?;
+        }
+
+        if self.remove {
+            fs::remove_file(&src_path).context("failed to remove")?;
         }
 
         Ok(())
@@ -68,7 +97,7 @@ fn move_to_data_dir(input_path: &Path) -> Result<PathBuf> {
     create_dir_all(format!("data/{}", hash_str)).context("failed to create `.data`")?;
 
     let to = PathBuf::from(format!("data/{}/input.js", hash_str));
-    fs::copy(input_path, &to).context("failed to copy")?;
+    fs::write(&to, src.as_bytes()).context("failed to write")?;
 
     Ok(to)
 }
